/**
 * Name: CollisionSphere.h
 *
 * Copyright (c) 2005 University of Bournemouth
 */
#ifndef __COLLISIONSPHERE_H__
#define __COLLISIONSPHERE_H__

#ifdef __APPLE__
    #include <GraphicsLib/Point3.h>
    #include <GraphicsLib/types.h>
    #include <GraphicsLib/Vector.h>
    #include <GraphicsLib/Util.h>
#include <GLUT/glut.h>
#else
#include <GL/glut.h>

    #include "Point3.h"
    #include "types.h"
    #include "Vector.h"
    #include "Util.h"
#endif

#include "Particle.h"
#include "CollisionObject.h"
#include "Debugger.h"



namespace ParticleSystems {
	//! The CollisionSphere Class
    /**
    * A class to create a sphere which can be used to make Particles collide against.
	* This Class extends the CollisionObject
	* @author Hannes Ricklefs
    */
	class CollisionSphere : public CollisionObject {
        public:
			//! The Constructor
			/**
			* Constructs a new CollisionSphere at the given point with the given
			* radius. 
			* \param c1 a pointer to the point at which to create the Sphere.
			* \param radius the radius of this CollisionSphere
			*/
            CollisionSphere( GraphicsLib::Point3 *c1, GraphicsLib::Real radius);
			
			//! The Destructor
			/**
			* Destructs the CollisionSphere
			*/
			virtual ~CollisionSphere();
			
            //! A virtual function
			/**
			* Checks if a collision occured between the passed Particle
			* and this CollisionSphere
			* \param p a pointer to the Particle
			*/
		    virtual void collide(Particle *p);
			
			//! A virtual function
			/**
			* Draws this CollisionSphere
			*/
            virtual void draw();
			
			//! A virtual function
			/**
			* Translates this CollisionSphere
			*/
            virtual void translate();
            
			//! A virtual function
			/**
			* Translates this CollisionSphere
			*/
			virtual void rotate();
            
			//! A virtual function
			/**
			* Translates this CollisionSphere
			*/
			virtual void scale();
			
			//! A public variable
			/**
			* The radius of this CollisionSphere
			*/
            GraphicsLib::Real radius;
			
			//! A public variable
			/**
			* The position of this CollisionSphere
			*/
			GraphicsLib::Point3 position;
			
	};
}
#endif