/**
 * Name: GravityForce.h
 *
 * Copyright (c) 2005 University of Bournemouth
 */
#ifndef __GRAVITYFORCE_H__
#define __GRAVITYFORCE_H__

#ifdef __APPLE__
#include <GraphicsLib/Vector.h>
#include <GraphicsLib/Util.h>
#else
#include "Util.h"
#include "Vector.h"
#endif

#include "Force.h"

namespace ParticleSystems {
	//! The GravityForce Class
	/**
	* The GravityForce Class extends the Force Class. It affects the 
	* direction of a Particle by appling the specified direction, 
	* attenuation and magnitude.
	* @author Hannes Ricklefs
	*/
	class GravityForce : public Force {
	public:
		//! The Constructor
		/**
		* The RandomForce Constructor takes a Vector that defines the direction
		* on how to influence the Particle accoriding to a given attenuation.
		* If the attenuation is set to 1 the Force will be fully applied
		* \param f a pointer to a direction Vector
		* \param a the attenuation
		* \param m the magnitude
		*/
		GravityForce(GraphicsLib::Vector *f, float a, float m);
		
		//! The Destructor
		/**
		* Destructs the GravityForce
		*/
		~GravityForce();
		
		//! A virtual Function
		/**
		* Applies this GravityForce to the given direction Vector.
		* \param direction The direction Vector to be affected by this GravityForce
		*/
		virtual void applyForce(GraphicsLib::Vector *direction);
		
		//! A virtual Function
		/**
		* Applies this GravityForce to the given direction Vector taking into consideration
		* a given Mass.
		* \param direction The direction Vector to be affected by this GravityForce
		* \param mass a pointer to the mass
		*/
		virtual void applyForce(GraphicsLib::Vector *direction, float *mass);
		
		//! A public Variable
		/**
		* The magnitude of the Gravity
		*/
		float magnitude;
	};
}
#endif