/**
 * Name: ParticleSystemManager.h
 *
 * Copyright (c) 2005 University of Bournemouth
 */
#ifndef __PARTICLESYSTEMMANAGER_H__
#define __PARTICLESYSTEMMANAGER_H__

#include "ParticleSystem.h"
#include "DataStructure.h"
#include "Debugger.h"

namespace ParticleSystems {

	//! The ParticleSystems within the scene
	/**
	* Stores all the ParticleSystems with in the scene
	*/
	typedef std::vector<ParticleSystem *> psList;

	//! The iterator for the ParticleSystems
	/**
	* Provides an iterator to perform actions on all the ParticleSystem.
	*/
	typedef std::vector<ParticleSystem *>::iterator psListItt;
	
	//! The ParticleSystemManager Class
	/**
	* The ParticleSystemManager is the main class for controlling all ParticleSystems within the scene.
	* It provides functionality for adding, removing and rendering ParticleSystems.
	* @author Hannes Ricklefs
	*/
	class ParticleSystemManager {

	public:
		//! The constructor
		/**
		* Constructs a ParticleSystemManager.
		*/
		ParticleSystemManager();
	
		//! The destructor
		/**
		*
		*/
		~ParticleSystemManager();
	
		//! A normal member function
		/**
		* Adds a new ParticleSystem.
		* \param particleSystem The new ParticlesSystem to add.
		*/
		void addSystem(ParticleSystem *particleSystem);
	
		//! A normal member function
		/**
		* Removes an existing ParticleSystem.
		* \param particleSystem The ParticlesSystem to be removed.
		*/
		void removeSystem(ParticleSystem *particleSystem);
	
		//! A normal member function 
		/**
		* Evolve all the ParticleSystems one evolution.
		*/
		void update();
	
		//! A normal member function
		/**
		* Render all the Particles within the scene
		*/
		void render();
	
		//! A normal member function
		/**
		* Stop and shutdown the ParticleSystemManager
		*/
		void reset();
		
		//! A normal member Function
		/**
		* Pause all the ParticleSystems
		*/
		void pause();
		
		//! A normal member Function
		/**
		* Restart all the ParticleSystems
		*/
		void restart();
	
		//! A normal member function
		/**
		* Returns the ID that would be assigned when adding
		* a new ParticleSystem 
		*/
		int getNextID();
		
		//! A normal member function
		/**
		* Returns if the specified ParticleSystem is controlled
		* by this ParticleSystemManager
		* \param particleSystem The ParticleSystem to be checked
		*/
		bool doesExist(ParticleSystem *particleSystem);
		
	private:
		//! The DataStructure
		/**
		* Stores the all the ParticleSystems within the scene.
		*/
		//DataStructure *dataStrc;
		psList pSystems;
	
		//! A private Variable
		/**
		* Stores the number of ParticleSystems controlled by this Manager.
		*/
		int numParticleSystems;
	
	
	};
}
#endif

