/**
 * Name: PointEmitter.h
 *
 * Copyright (c) 2005 University of Bournemouth
 */
#ifndef __POINTEMITTER_H__
#define __POINTEMITTER_H__

#ifdef __APPLE__
#include <GraphicsLib/Point3.h>
#include <GraphicsLib/Util.h>
#include <GLUT/glut.h>
#else
#include <GL/glut.h>
#include "Util.h"
#include "Point3.h"
#endif

#include "Emitter.h"
#include "PointParticle.h"
#include "ImgParticle.h"
#include "StrikeParticle.h"

namespace ParticleSystems {
	//! The PointEmitter Class
	/**
	* The PointEmitter is a sub class of the Emitter. It emits Particles
	* form a specified Point in 3D space. It also has a sweep angle so a spherical
	* shape can be accieved.
	* @author Hannes Ricklefs
	*/
	class PointEmitter : public Emitter {
	public:
		
		//! The Constructor
		/**
		* Constructs a new Emitter
		* \param SYSID the ParticleSystemID
		* \param type the ParticleType
		* \param initVel the initial velocity for a particle
		* \param initalColor the inital Colour of a particle
		* \param pSize the inital Size
		* \param initPos the position in 3D space
		* \param sweep the sweep angle for emition
		*/
		PointEmitter(	int SYSID, 
						Solver *s, 
						PARTICLETYPE type, 
						GraphicsLib::Vector initVel, 
						GraphicsLib::Colour initialColor, 
						GLfloat pSize,
						GraphicsLib::Point3 initPos,
						GLfloat sweep); 
						
		//! The Constructor
		/**
		* Constructs a new Emitter with a Texture
		* \param SYSID the ParticleSystemID
		* \param type the ParticleType
		* \param initVel the initial velocity for a particle
		* \param initalColor the inital Colour of a particle
		* \param pSize the inital Size
		* \param TexName a pointer to a texture
		* \param initPos the inital position in 3D
		* \param sweep the emition angle
		*/
		PointEmitter(	int SYSID, 
						Solver *s, 
						PARTICLETYPE type, 
						GraphicsLib::Vector initVel, 
						GraphicsLib::Colour initialColor, 
						GLfloat pSize,
						char *TexName, 
						GraphicsLib::Point3 initPos,
						GLfloat sweep);
		
		//! The Destructor
		/**
		* Destructs this Emitter
		*/
		~PointEmitter();
		
		//! An virtual Function
		/**
		* Emitts a new Particle according to the values
		* \return A pointer to the new Particle
		*/
		virtual Particle* emittParticles(void);
		
		//! An virtual Function
		/**
		* Reinitialises the given Particle as it would be newly created
		* \param P a pointer to the Particle to be reborne
		*/
		virtual void reBirth(Particle *p);
		
		//! A member Function
		/**
		* Returns a vector with the emition angle added
		* \return the vector
		*/
		GraphicsLib::Vector withSweep();
		
		//! A public variable
		/**
		* The emittion position
		*/
		GraphicsLib::Point3 initPosition;
		
		//! A public variable
		/**
		* The sweep emition angle
		*/
		GLfloat sweep;
	};
}
#endif


