/**
 * Name: Solver.h
 *
 * Copyright (c) 2005 University of Bournemouth
 */
#ifndef __SOLVER_H__
#define __SOLVER_H__

#ifdef __APPLE__
#include <GraphicsLib/Vector.h>
#include <GraphicsLib/Point3.h>
#else
#include "Point3.h"
#include "Vector.h"
#endif

#include <vector>
#include <iostream>
#include "Force.h"


namespace ParticleSystems {
	//! The Solver Class
	/**
	* The Solver Class main functionality is to evolve a Particles direction
	* accoriding to its specified rules. The Solver Class is the Super Class and 
	* all the inheriting classes need to implement the abstract methods. A Solver 
	* updates the particle movement using the given forces.
	* @author Hannes Ricklefs
	*/
	class Solver {
		public:
			
			//! The Constructor
			/**
			* The Constructor which adds the given Force to the
			* forces vector.
			* \param f a pointer to the Force
			*/
			Solver(Force *f);
			
			//! The default Constructor
			/**
			* The default Constructor
			*/
			Solver();
			
			//! The Destructor
			/**
			* The Destructor
			*/
			virtual ~Solver();	
			
			//! A public Function
			/**
			* Adds a Force to this Solver to affect the 
			* Particle
			* \param f a pointer to the Force to be added
			*/
			void addForce(Force *f);
			
			//! A public Function
			/**
			* Removes a Force from this Solver
			* \param f a pointer to the Force to be removed
			*/
			void removeForce(Force *f);
			
			//! A public Function
			/**
			* Prints all the Forces to stdout.
			*/
			void listForces();
			
			//! An abstract Function
			/**
			* Evolves the given direction Vector by applying the
			* Forces assigned to this Solver. 
			* \param direction a pointer to the direction Vector to change
			*/
			virtual void nextStep(GraphicsLib::Vector *direction) = 0;
			
			//! An abstract Function
			/**
			* Evolves the given direction Vector by applying the
			* Forces assigned to this Solver. This function also takes
			* into consideration the mass passed.
			* \param direction a pointer to the direction Vector to change
			* \param mass the mass of the Particle
			*/
			virtual void nextStep(GraphicsLib::Vector *direction, float *mass) = 0;
			
			//! A public variable
			/**
			* The Force affecting the Particle movement.
			*/
			std::vector<Force *> forces;
			
			//! A public variable
			/**
			* An Itterator for the Forces
			*/
            std::vector<Force *>::iterator forcesItt;
			
			//! A public variable
			/**
			* The duration of this Solver
			*/
			int duration;
			
	};
}
#endif

