/**
 * Name: StrikeParticle.h
 *
 * Copyright (c) 2005 University of Bournemouth
 */
#ifndef __STRIKEPARTICLE_H__
#define __STRIKEPARTICLE_H__

#ifdef __APPLE__
	#include <GLUT/glut.h>
#else
	#include <GL/glut.h>
#endif

#include "Particle.h"
#include "Debugger.h"

namespace ParticleSystems {
	//! The PointParticle Class
	/**
	* The PointParticle Class is a Subclass of the Particle Class.
	* It extends the Particle Class in order to be able to draw
	* a Strike (Line) as a Particle.
	* @see Particle
	* @author Hannes Ricklefs
	*/
	class StrikeParticle : public Particle {
	public:
        
		//! The minimal Constructor
		/**
		* This Constructor only takes the ParticleSystem ID and the 
		* Solver.
		* \param sysId the ParticleSystem ID this PointParticle belongs to
		* \param sol a pointer to the Solver
		*/
		StrikeParticle(int sysId, Solver *sol);
		
		//! The main Constructor
		/**
		* This Constructor is the main Constructor for a PointParticle. It sets the 
		* most important attributes of a PointParticle. 
		* \param sysId the ParticleSystem ID this Particle belongs to
		* \param sol a pointer to the Solver
		* \param pos the initial Position
		* \param vel the inital Velocity
		* \param c the initial Colour
		* \param fc the fade Colour
		* \param size the inital Size
		* \param fsize the fade Size
		* \param maxLife the maximum Lifespan of the Particle
		*/
		StrikeParticle(int sysId, 
					Solver *sol, 
					GraphicsLib::Point3 pos, 
					GraphicsLib::Vector vel, 
					GraphicsLib::Colour c,
					GraphicsLib::Colour fc,
					GLfloat size,
					GLfloat fs,
					GLfloat maxLife);
		//! The Destructor
		/**
		* Destructs the PointParticle
		*/
		virtual ~StrikeParticle();		
		
		//! A virtual Function
		/**
		* Draws the PointParticle
		*/
		virtual void draw(void);
		
		//! A virtual Function
		/**
		* Updates the PointParticles
		* attributes 
		*/
		virtual void update(void);
		
		//! A virtual Function
		/**
		* Returns true if the PointParticle is dead
		* \return the value if the PointParticle is dead
		*/
		virtual bool isDead(void);		
		
	};
}
#endif
