/**
 * Name: TGALoader.h
 *
 * Copyright (c) 2005 University of Bournemouth
 */
#ifndef __TGA_H__
#define __TGA_H__
#include <stdio.h>										// Standard I/O header
#include <iostream>
#ifdef __APPLE__
#include <OpenGL/gl.h>
#else
	#include <GL/glut.h>
#endif
#include "Texture.h"

/*
* This file came from Alex Klaminsky it has been changed to 
* fit into this implementation.
* @author Hannes Ricklefs
*/

namespace ParticleSystems {

//! A struct
/**
* The TGA File Header
*/
typedef struct
{
	GLubyte Header[12];									
} TGAHeader;

//! A struct
/**
* This struct enables handeling of TGA files
*/
typedef struct
{
	/** First 6 Useful Bytes From The Header */
	GLubyte		header[6];								
	/** Holds Number Of Bytes Per Pixel Used In The TGA File */
	GLuint		bytesPerPixel;							 
	/** Used To Store The Image Size When Setting Aside Ram */
	GLuint		imageSize;								 
	/** Temporary Variable */
	GLuint		temp;									 
	GLuint		type;
	/** Height of Image */
	GLuint		Height;									
	/** Width ofImage */
	GLuint		Width;									 
	/** Bits Per Pixel */
	GLuint		Bpp;									
} TGA;

//! The TGALoader Class
/**
* This class enables TGA picture file handeling. 
* It enables loading and binding a TGA image to a texture.
*/
class TGALoader {

	public:
		//! The Constructor
		/**
		* Constructs a new TGALoader
		*/
		TGALoader();
		
		//! The Destructor
		/**
		* Destructs a TGALoader
		*/
		~TGALoader();
		
		//! A member function
		/**
		* Open and test the file to make sure it is a valid TGA file
		* \param texture a pointer to the texture
		* \param filename a pointer to the filename 
		*/		
		bool LoadTGA(Texture * texture, char * filename);
		
		//! A public variable
		/**
		* The TGA header
		*/
		TGAHeader tgaheader;									
		
		//! A public variable
		/**
		* The TGA image data
		*/
		TGA tga;	
		
		//! A public variable
		/**
		* Uncompressed TGA Header
		*/
		GLubyte uTGAcompare[12];	
		
		//! A public variable
		/**
		* Compressed TGA Header
		*/
		GLubyte cTGAcompare[12]; 
	
	private:
		//! A member function
		/**
		* Load a Uncompressed file return true if succesfully loaded.
		* \param Texture a pointer to the texture
		* \param char a pointer to a char
		* \param FILE a pointer to the file to be loaded
		*/
		bool LoadUncompressedTGA(Texture *, char *, FILE *);	
		
		//! A member function
		/**
		* Load a Compressed file return true if succesfully loaded.
		* \param Texture a pointer to the texture
		* \param char a pointer to a char
		* \param FILE a pointer to the file to be loaded
		*/
		bool LoadCompressedTGA(Texture *, char *, FILE *);	
	
};
}
#endif






