/**
 * Name: ImgParticle.cpp
 *
 * Copyright (c) 2005 University of Bournemouth
 */
#include "ImgParticle.h"
#ifndef __APPLE__
#include <tiffio.h>
#endif

using namespace GraphicsLib;
using namespace ParticleSystems;


ImgParticle::ImgParticle(int sysId, Solver *sol, Texture *tex) : Particle(sysId,sol) {
	texture = tex;
}

ImgParticle::ImgParticle(	int sysId, 
							Solver *sol, 
							Point3 pos, 
							Vector vel, 
							Colour c, 
							Colour fc,
							GLfloat s,
							GLfloat fs, 
							GLfloat ml, 
							Texture *tex) : Particle(sysId, sol, pos, vel, c,fc,s ,fs ,ml) {
	texture = tex;
}

ImgParticle::~ImgParticle() {
}

void ImgParticle::draw(void) {
    #ifdef __PERFORMANCE__
        std::cout << "IMGParticle: beforedrawing particles" << std::endl;
    #endif


	// used for billboarding
	float matrix[16];

	glGetFloatv(GL_MODELVIEW_MATRIX,matrix);

	float x0 = matrix[0] - matrix[1];
	float x1 = matrix[0] + matrix[1];

	float y0 = matrix[4] - matrix[5];
	float y1 = matrix[4] + matrix[5];

	float z0 = matrix[8] - matrix[9];
	float z1 = matrix[8] + matrix[9];
	glPushMatrix();
			// create the quad to apply the Texture
			float s  = size;
			float x  = position.x;
			float y  = position.y;
			float z  = position.z;
			float xx = old_Position.x;
			float yy = old_Position.y;
			float zz = old_Position.z;

			// define the particles colour
			colour.Use();
	

			glBegin(GL_QUADS);
				glTexCoord2d(1,0);glVertex3f(xx - x1*s,yy - y1*s,zz - z1*s);
				glTexCoord2d(1,1);glVertex3f(x + x0*s,y + y0*s,z + z0*s);
				glTexCoord2d(0,1);glVertex3f(x + x1*s,y + y1*s,z + z1*s);
				glTexCoord2d(0,0);glVertex3d(xx - x0*s,yy - y0*s,zz - z0*s);
			glEnd();
		
	
	glPopMatrix();


    #ifdef __PERFORMANCE__
        std::cout << "IMGParticle: afterdrawing particles" << std::endl;
    #endif
}

void ImgParticle::update(void) {
	
	//use a temp Vector
	Vector *tmp = &velocity;
	solv->nextStep(tmp);
	#ifdef __COLLISION__
            std::cout << "In particle " << position<< std::endl;
	#endif
	//update every attribute
	old_Position = position;
	position+=velocity;
	size -= flare_size;
	colour = colour - faded_colour;
	life+=1.0;
	if(life>MAXLIFE) {
		dead = true;
	}
	#ifdef __COLLISION__
            std::cout << "In particle new" << position<< std::endl;
	#endif
}

bool ImgParticle::isDead() {
	return dead;
}



