/**
 * Name: ParticleSystem.cpp
 *
 * Copyright (c) 2005 University of Bournemouth
 */
#include "ParticleSystem.h"
#include <iostream>
using namespace std;
using namespace GraphicsLib;
using namespace ParticleSystems;


ParticleSystem::ParticleSystem(int ident, Emitter *emit, DataStructure *dataStruc, Solver *s, int num, int var) {
	identify = ident;
	alive = true;
	paused = false;
	position = new Point3(0,0,0);
	em = emit;
	data = dataStruc;
	numInitParticles = num;
	varNewParticles = var;
	addSolver(s);
	
	//set up the initial system
	this->createNewParticles(numInitParticles);
	//set the storage capacity of the dead particles
	data->storageCapacity = var;
}

ParticleSystem::~ParticleSystem() {
	delete em;
	delete data;
}

/*!
 * Kill the entire ParticleSystem
 */
void ParticleSystem::die() {
	alive = false;
	killParticles();
}

void ParticleSystem::restart() {
	alive = true;
}

void ParticleSystem::pause() {
	paused ^= true;
}

/*
 * Returns if the system is still alive
 */
bool ParticleSystem::stillAlive() {
	return alive;
}

/*
 * Creates a new particle
 */
void ParticleSystem::newParticle() {
	//tell the emitter to emit new particles
    //cout << "in new Partilce" << endl;
	//check if dead Particles are available
	if(data->hasDead()) {
        //if(data->getDead() !=NULL)
        #ifdef __PERFORMANCE__
        cout << "ParticleSystem: rebirth" << endl;
        #endif
        em->reBirth(data->getDead());
	} else {
		data->addParticles(em->emittParticles());
        #ifdef __PERFORMANCE__
        cout << "ParticleSystem: adding" << endl;
        #endif
	}
}

/*
 * Kill the particles
 */
void ParticleSystem::killParticles() {
	data->killParticles();
}

/*
* Evolves the particle system adding new particles accoring to the birthrate
*/
void ParticleSystem::evolve() {
	//evolve and a new Particles
	if(!paused && alive) {
		//Random value for the new emition
		int var = RandomPosNum(varNewParticles);
		#ifdef __PARTICLE_GENERATION__
		std::cout << var << std::endl;
		#endif
		for(int i = 0; i < var; i++) {
			this->newParticle();
		}
		//evolve the existing Particles
		data->evolve();
	}

}

void ParticleSystem::createNewParticles(int num) {
	for(int i = 0; i < num; i++) {
		this->newParticle();
	}
}
/*
* Add a new solver
*/
void ParticleSystem::addSolver(Solver *s) {
	sList.insert(sList.begin(), s);

}

void ParticleSystem::draw() {
	//if type is IMAGE do the texture binding before
	//not once per particle
    if(em->type == Emitter::IMAGE) {
        glGenTextures(1, &((em->texture).texID));
	    glBindTexture(GL_TEXTURE_2D, (em->texture).texID);
	    glTexImage2D(GL_TEXTURE_2D, 0, 3, (em->texture).width, (em->texture).height, 0, GL_RGB,
					GL_UNSIGNED_BYTE, (em->texture).imageData);
	    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
	    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);

	    glDisable(GL_LIGHTING);
	    glEnable(GL_TEXTURE_2D);
	    glBlendFunc(GL_SRC_ALPHA,GL_ONE);
	    glEnable(GL_BLEND);
        data->draw();
        glDisable(GL_TEXTURE_2D);
	    glDisable(GL_BLEND);
    } else {
		//just draw with DEPTH_TEST enabled
		//if we draw image we need it switched off
		glEnable(GL_LIGHTING);
        glEnable(GL_DEPTH_TEST);
        data->draw();
        glDisable(GL_DEPTH_TEST);
    }
}




