/**
 * Name: ParticleSystemManager.cpp
 *
 * Copyright (c) 2005 University of Bournemouth
 */
#include "ParticleSystemManager.h"

using namespace ParticleSystems;

ParticleSystemManager::ParticleSystemManager(){
	 numParticleSystems = 0;
}


void ParticleSystemManager::addSystem(ParticleSystem *particleSystem) {
	//add the system to the psList vector
	pSystems.insert(pSystems.begin(), particleSystem);
	//increase the number of systems in controlled by the manager
	numParticleSystems++;
}

void ParticleSystemManager::removeSystem(ParticleSystem *particleSystem) {
	//find the given Force.
	std::vector<ParticleSystem *>::iterator findPS = std::find(pSystems.begin(),pSystems.end(),particleSystem);
	//If it matches the last element we know that
	//it didnt find the given force.
	if(findPS != pSystems.end()) {
		pSystems.erase(findPS);
	}
}

void ParticleSystemManager::reset() {
	psListItt itt = pSystems.begin();
	psListItt end = pSystems.end();
	while(itt != end) {
		(*itt)->die();
		itt++;
	}
}

void ParticleSystemManager::pause() {
	psListItt itt = pSystems.begin();
	psListItt end = pSystems.end();
	while(itt != end) {
		(*itt)->pause();
		itt++;
	}
}

void ParticleSystemManager::restart() {
	psListItt itt = pSystems.begin();
	psListItt end = pSystems.end();
	while(itt != end) {
		(*itt)->restart();
		itt++;
	}
}

void ParticleSystemManager::update() {
	//add new particles
	psListItt itt = pSystems.begin();
	psListItt end = pSystems.end();
	
    #ifdef __PERFORMANCE__
    std::cout << "ParticleSystemManager: Before while->evolve"  << std::endl;
    #endif
	
	while(itt != end) {
		(*itt)->evolve();
		itt++;
	}
	
    #ifdef __PERFORMANCE__
    std::cout << "ParticleSystemManager: After while-> evolve"  << std::endl;
    #endif
	
	#ifdef  __PARTICLE_GENERATION__
	std::cout << "ParticleSystemManager: "<< dataStrc->size()  << std::endl;
	#endif
}

void ParticleSystemManager::render() {
    psListItt itt = pSystems.begin();
	psListItt end = pSystems.end();
    while(itt != end) {
		//draw each Particle System
		(*itt)->draw();
		itt++;
	}
}

int ParticleSystemManager::getNextID() {
	return (numParticleSystems+1);
}

bool ParticleSystemManager::doesExist(ParticleSystem *particleSystem) {
	return true;
}

