
/**
 * Name: PlanarEmitter.cpp
 *
 * Copyright (c) 2005 University of Bournemouth
 */
#include "PlanarEmitter.h"
using namespace GraphicsLib;
using namespace ParticleSystems;


PlanarEmitter::PlanarEmitter(	int SYSID, 
								Solver *s, 
								PARTICLETYPE type, 
								Vector initVel, 
								Colour initialColor, 
								GLfloat pSize, 
								Vector dim1, 
								Vector dim2) : Emitter(SYSID, s, type, initVel, initialColor, pSize) {
	dimension1 = dim1;
	dimension2 = dim2;
} 

PlanarEmitter::PlanarEmitter(	int SYSID, 
								Solver *s, 
								PARTICLETYPE type, 
								Vector initVel, 
								Colour initialColor, 
								GLfloat pSize,
								char *TexName, 
								Vector dim1, 
								Vector dim2) : Emitter(SYSID, s, type, initVel, initialColor, pSize, TexName) {
	dimension1 = dim1;
	dimension2 = dim2;
}

PlanarEmitter::~PlanarEmitter() {
}

Particle* PlanarEmitter::emittParticles() {
	//The result according to the type
	Particle *result;
	if(type == POINT) {
		#ifdef __SPRITE__
            std::cout << "Planar emitter: Emitting Sphere" << std::endl;
		#endif
		result = new PointParticle(	SYSID,
									solver,
									pointOnPlane(),
									initialVelocity,
									initialColor,
									flareColor,
									particleSize, 
									particleSizeFlare,
									RandomPosNum(particleLife));
	} else if(type == IMAGE) {
		#ifdef __SPRITE__
            std::cout << "Planar Emmiter: Emitting Sprite" << std::endl;
		#endif
		result = new ImgParticle(	SYSID,
									solver,
									pointOnPlane(), 
									initialVelocity, 
									initialColor,
									flareColor,
									particleSize, 
									particleSizeFlare,
									RandomPosNum(particleLife),
									&texture);
	} else if(type == STRIKE) {
		#ifdef __SPRITE__
            std::cout << "PointEmmitter: Emitting Sphere" << std::endl;
		#endif
		
		result = new StrikeParticle(SYSID,
									solver,
									pointOnPlane(),
									initialVelocity,
									initialColor, 
									flareColor,
									particleSize,
									particleSizeFlare, 
									RandomPosNum(particleLife));
	}
	
	return result;

}

void PlanarEmitter::reBirth(Particle *p) {
	p->position = pointOnPlane();
	p->life = 0;
	p->dead = false;
	p->size = particleSize;
	p->velocity= initialVelocity;;
	p->colour = initialColor;
	p->faded_colour = flareColor;
}


Point3 PlanarEmitter::pointOnPlane() {
	Real randomU = RandomPosNum(1.0f);
	Real randomV = RandomPosNum(1.0f);
	//Needed to return a point when adding two vectors
	Point3 neutral = new Point3(0,0,0);
	Point3 result = (neutral + (randomU * dimension1) + (randomV * dimension2));
	return result;

}
