/**
 * Name: PointEmitter.cpp
 *
 * Copyright (c) 2005 University of Bournemouth
 */
#include "PointEmitter.h"
#include "StraightSolver.h"
#include "UniformForce.h"

using namespace GraphicsLib;
using namespace ParticleSystems;

PointEmitter::PointEmitter(	int SYSID, 
							Solver *s,  
							PARTICLETYPE type, 
							Vector initVel, 
							Colour initialColor, 
							GLfloat pSize,
							Point3 initPos, 
							GLfloat sw) : Emitter(SYSID, s, type, initVel, initialColor, pSize) {
	initPosition = initPos;
	sweep = sw;
}

PointEmitter::PointEmitter(	int SYSID, 
							Solver *s, 
							PARTICLETYPE type, 
							Vector initVel, 
							Colour initialColor, 
							GLfloat pSize, 
							char *TexName, 
							Point3 initPos,
							GLfloat sw) : Emitter(SYSID, s, type, initVel, initialColor, pSize, TexName){
	initPosition = initPos;
	sweep = sw;
}

PointEmitter::~PointEmitter() {
}


Particle* PointEmitter::emittParticles(void) {
	//Create a new Particle depending on its type
	Particle *result;
	if(type == POINT) {
		#ifdef __SPRITE__
            std::cout << "PointEmmitter: Emitting Sphere" << std::endl;
		#endif
		result = new PointParticle(	SYSID,
									solver,
									initPosition,
									withSweep(),
									initialColor, 
									flareColor,
									particleSize,
									particleSizeFlare, 
									RandomPosNum(particleLife));
	} else if(type == IMAGE) {
		#ifdef __SPRITE__
            std::cout << "PointEmmitter: Emitting Sprite" << std::endl;
		#endif
		result = new ImgParticle(	SYSID,
									solver,
									initPosition,
									withSweep(),
									initialColor,
									flareColor,
									particleSize, 
									particleSizeFlare,
									RandomPosNum(particleLife),
									&texture);
	} else if(type == STRIKE) {
		#ifdef __SPRITE__
            std::cout << "PointEmmitter: Emitting Sphere" << std::endl;
		#endif
		result = new StrikeParticle(	SYSID,
									solver,
									initPosition,
									withSweep(),
									initialColor, 
									flareColor,
									particleSize,
									particleSizeFlare, 
									RandomPosNum(particleLife));
	}
	
	return result;
}

Vector PointEmitter::withSweep() {
	//The result
	Vector result = initialVelocity;
	//Add the sweep randomly to each direction.
	//if(result.x == 0)
	result.x += RandomNum(sweep);
	//if(result.y == 0)
	result.y += RandomNum(sweep);
	//if(result.z == 0)
	result.z += RandomNum(sweep);
	return result;
}

void PointEmitter::reBirth(Particle *p) {
	//reinitialise the dead Particle
	#ifdef __PERFORMANCE__
    std::cout << "PointEmiter: Before setting rebirth"  << std::endl;
    #endif
	p->position = initPosition;
	p->size = particleSize;
	p->colour = initialColor;
	p->faded_colour = flareColor;
	p->life = 0;
	p->dead = false;
	p->velocity=withSweep();
	#ifdef __PERFORMANCE__
    std::cout << "PointEmiter: after setting rebirth"  << std::endl;
    #endif
}
